@@ -3,7 +3,7 @@ import logging
 import os
 import posixpath
 import urllib.parse
-from typing import List
+from typing import Dict, List, Optional
 
 from pipenv.patched.notpip._vendor.packaging.utils import canonicalize_name
 
@@ -20,13 +20,14 @@ class SearchScope:
     Encapsulates the locations that pip is configured to search.
     """
 
-    __slots__ = ["find_links", "index_urls"]
+    __slots__ = ["find_links", "index_urls", "index_lookup"]
 
     @classmethod
     def create(
         cls,
         find_links: List[str],
         index_urls: List[str],
+        index_lookup: Optional[Dict[str, List[str]]] = None,
     ) -> "SearchScope":
         """
         Create a SearchScope object after normalizing the `find_links`.
@@ -60,15 +61,18 @@ class SearchScope:
         return cls(
             find_links=built_find_links,
             index_urls=index_urls,
+            index_lookup=index_lookup
         )
 
     def __init__(
         self,
         find_links: List[str],
         index_urls: List[str],
+        index_lookup: Optional[Dict[str, List[str]]] = None,
     ) -> None:
         self.find_links = find_links
         self.index_urls = index_urls
+        self.index_lookup = index_lookup if index_lookup else {}
 
     def get_formatted_locations(self) -> str:
         lines = []
@@ -123,4 +127,7 @@ class SearchScope:
                 loc = loc + '/'
             return loc
 
-        return [mkurl_pypi_url(url) for url in self.index_urls]
+        index_urls = self.index_urls
+        if project_name in self.index_lookup:
+            index_urls = [self.index_lookup[project_name]]
+        return [mkurl_pypi_url(url) for url in index_urls]
